@@ -29,6 +29,7 @@ __all__ = ["unpack_uint12", "fvb", "fvb_images", "decode_image_data", "decode_im
 import logging
 from threading import Thread, Event
 import struct
+from time import monotonic, sleep
 
 import numpy as np
 
@@ -382,12 +383,17 @@ class FrameServer():
     If ``fvb=True``, then the data provided to the callback method is a 1-dimensional numpy array.
     Otherwise, the data is a 2-dimensional numpy array with axes (row, column).
 
+    The rate that frames are served can be limited using the ``frame_rate_max`` parameter.
+    A value of zero or ``None`` will not restrict the frame rate. Note that this is the rate
+    of frames served, not the actual acquisition rate.
+
     :param cam: :class:`~andor3.andor3.Andor3` camera to use for frame acquisition.
     :param frame_callback: Function to call on each frame acquisition event.
     :param completion_callback: Function to call when acquisition is completed.
     :param fvb: Perform full-vertical-binning on the image data.
+    :param frame_rate_max: Maximum frame serving rate in frames per second.
     """
-    def __init__(self, cam, frame_callback, completion_callback=None, fvb=False):
+    def __init__(self, cam, frame_callback, completion_callback=None, fvb=False, frame_rate_max=None):
         self._cam = cam
         self._thread = None
         self._thread_stop = Event()
@@ -402,8 +408,9 @@ class FrameServer():
                 _log.warning("FrameServer completion_callback parameter is not a callable function, ignoring.")
             self._completion_callback = None
         self._fvb = bool(fvb)
+        self._frame_interval = 1.0/frame_rate_max if frame_rate_max else 0.0
 
-    def start(self, nbuffers=10, fvb=None):
+    def start(self, nbuffers=10, fvb=None, frame_rate_max=None):
         """
         Start the camera and begin serving acquired frames.
 
@@ -415,12 +422,19 @@ class FrameServer():
 
         Full-vertical binning can be enabled by setting ``fvb=True``.
 
+        The rate that frames are served can be limited using the ``frame_rate_max`` parameter.
+        A value of zero or ``None`` will not restrict the frame rate. Note that this is the rate
+        of frames served, not the actual acquisition rate.
+
         :param nbuffers: Number of frames in the circular image buffer.
         :param fvb: Perform full-vertical-binning on the image data.
+        :param frame_rate_max: Maximum acquisition rate in frames per second.
         """
         self._cam.queueBuffer(nbuffers)
         if fvb is not None:
             self._fvb = bool(fvb)
+        if frame_rate_max is not None:
+            self._frame_interval = 1.0/frame_rate_max if frame_rate_max else 0.0
         self._thread_stop.clear()
         self._thread = Thread(target=self._run, name="Andor3_FrameServer", daemon=True)
         self._thread.start()
@@ -435,9 +449,12 @@ class FrameServer():
             self._thread.join()
 
     def _run(self):
-        # Camera does not automatically stop acquiring once FrameCount has been reached
+        """
+        Run the acquisition thread.
+        """
         frame_count = 0
         self._cam.CycleMode = "Continuous"
+        last_frame_time = 0.0
         self._cam.start()
         while (not self._thread_stop.is_set()) and self._cam.CameraAcquiring:
             # Requeue to use as a circular buffer
@@ -452,12 +469,19 @@ class FrameServer():
                     break
                 else:
                     raise
+            # Rate limit the frame serving if a maximum frame rate was specified
+            if self._frame_interval:
+                this_frame_time = monotonic()
+                if this_frame_time < last_frame_time + self._frame_interval:
+                    continue
             # Notify listener a frame has been acquired
             img, timestamp = self._cam.decode_image(data)
             if self._fvb:
                 img = fvb(img)
             self._frame_callback(frame_count, img, timestamp)
             frame_count += 1
+            if self._frame_interval:
+                last_frame_time = monotonic()
         # Stop the acquisition if it isn't already
         self._cam.stop()
         # Notify listener the acquisition is completed
